import json
from collections.abc import Sequence
from dataclasses import dataclass
from time import sleep

import invoke as inv

from .str_parsing import get_letter_alphabet_position, get_letter_from_alphabet_position


@dataclass(frozen=True)
class GithubIssue:
    number: int
    title: str


def get_issues_assigned_to_me(c: inv.Context) -> Sequence[GithubIssue] | None:
    """Get issues assigned to current user on current repo"""
    my_issues_cmd = c.run("gh issue list --assignee='@me' --json number,title", hide="out")

    if my_issues_cmd is None:
        return None

    parsed_output = [GithubIssue(**value) for value in json.loads(my_issues_cmd.stdout)]
    return parsed_output


def issue_dialog(my_issues: Sequence[GithubIssue]) -> int:
    issue_strings = [
        f"[{get_letter_from_alphabet_position(i+1)}] #{issue.number} {issue.title}"
        for i, issue in enumerate(my_issues)
    ]

    n_issues = len(my_issues)
    if n_issues == 1:
        print(f"Only one issue found, selecting:\n\t{issue_strings[0]}\nin ")
        for i in range(1 * 10, 0, -1):
            interval = 0.1
            print(f"{round(i * interval, 1)}...")
            sleep(interval)
        return 0

    terminal_output = "\n".join(reversed(issue_strings))
    print(f"\n{terminal_output}\n")

    print(f"Latest [a] is: \n\t {my_issues[0].title}\n")
    print("I found these issues for you. Which do you want to work on?\n")

    issue_index = (
        get_letter_alphabet_position(input(f"[a-{get_letter_from_alphabet_position(n_issues)}]: "))
        - 1
    )

    return issue_index


def sanitise_issue_title(issue_title: str) -> str:
    characters_to_replace = [" ", ":", ",", "'", '"', "(", ")", "[", "]", "`"]

    for character in characters_to_replace:
        issue_title = issue_title.replace(character, "-")

    issue_title = issue_title.replace("--", "-")
    return issue_title



from pathlib import Path

from memium.destination.ankiconnect.anki_converter import AnkiPromptConverter
from memium.destination.ankiconnect.ankiconnect_gateway import ANKICONNECT_URL, AnkiConnectGateway
from memium.destination.destination import PushPrompts
from memium.destination.destination_ankiconnect import AnkiConnectDestination
from memium.destination.destination_dryrun import DryRunDestination
from memium.diff_determiner import PromptDiffDeterminer
from memium.environment import host_input_dir, in_docker
from memium.source.document_source import MarkdownDocumentSource
from memium.source.extractors.extractor_qa import QAPromptExtractor
from memium.source.extractors.extractor_table import TableExtractor
from memium.source.prompt_source import DocumentPromptSource


def main(
    base_deck: str,
    input_dir: Path,
    max_deletions_per_run: int,
    dry_run: bool,
    push_all: bool = False,
):
    source_prompts = DocumentPromptSource(
        document_ingester=MarkdownDocumentSource(directory=input_dir),
        prompt_extractors=[
            QAPromptExtractor(question_prefix="Q.", answer_prefix="A."),
            TableExtractor(),
        ],
    ).get_prompts()

    dest_class = AnkiConnectDestination if not dry_run else DryRunDestination
    destination = dest_class(
        gateway=AnkiConnectGateway(
            ankiconnect_url=ANKICONNECT_URL,
            base_deck=base_deck,
            tmp_read_dir=host_input_dir() if in_docker() else input_dir,
            tmp_write_dir=input_dir,
            max_deletions_per_run=max_deletions_per_run,
            max_wait_seconds=30,
        ),
        prompt_converter=AnkiPromptConverter(
            base_deck=base_deck,
            card_css=Path("memium/destination/ankiconnect/default_styling.css").read_text(),
        ),
    )

    update_commands = (
        [PushPrompts(prompts=source_prompts)]
        if push_all
        else PromptDiffDeterminer().sync(
            source_prompts=source_prompts, destination_prompts=destination.get_all_prompts()
        )
    )

    destination.update(commands=update_commands)



import pytest

from ...source.prompts.prompt import BasePrompt
from ...source.prompts.prompt_cloze import ClozeWithoutDoc
from ...source.prompts.prompt_qa import QAWithoutDoc
from .anki_converter import AnkiPromptConverter
from .anki_prompt import AnkiPrompt
from .test_anki_prompt_qa import FakeAnkiCloze, FakeAnkiQA


@pytest.mark.parametrize(
    ("input_prompt", "expected_card"),
    [
        (
            QAWithoutDoc(question="FakeQuestion", answer="FakeAnswer", add_tags=["FakeTag"]),
            FakeAnkiQA(
                uuid=4875918425
            ),  # Ensure that UUID generation remains stable to retain idempotency over time
        ),
        (
            ClozeWithoutDoc(text="FakeText", add_tags=["FakeTag"]),
            FakeAnkiCloze(
                uuid=1873301177
            ),  # Ensure that UUID generation remains stable to retain idempotency over time
        ),
    ],
)
def test_anki_prompt_converter(input_prompt: BasePrompt, expected_card: AnkiPrompt):
    """Tests the AnkiPromptConverter class"""
    generated_card = AnkiPromptConverter(
        base_deck="FakeBaseDeck", card_css="FakeCSS"
    ).prompt_to_card(input_prompt)

    assert generated_card.uuid == expected_card.uuid
